/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "Condition.h"
#include "Game.h"

static const basic_string <char>::size_type npos = -1;

Condition::Condition(long type)
:	fType(kConditionTypeIP),
	fDataType(kConditionDataTypeLong),
	fOperation(kConditionOpEqual),
	fLongValue(0),
	fFloatValue(0.0),
	fStringValue("")
{
	switch (type)
	{
	case kConditionTypeIP:
	case kConditionTypePort:
	case kConditionTypeCntPlayers:
	case kConditionTypeCustomLong:
		fDataType = kConditionDataTypeLong;
		break;
	case kConditionTypeCustomFloat:
		fDataType = kConditionDataTypeFloat;
		break;
	case kConditionTypeHostname:
	case kConditionTypeGame:
	case kConditionTypeGamedir:
	case kConditionTypeGametype:
	case kConditionTypeMap:
	case kConditionTypePlayer:
	case kConditionTypeCustomString:
		fDataType = kConditionDataTypeString;
		break;
	}
}

void Condition::SetValue(unsigned long value, long operation)
{
	if (fDataType != kConditionDataTypeLong)
		return;
	if (operation >= 0 && (operation & 255) <= kConditionOpGreater)
	{
		fOperation = (operation & kConditionOpNot) | (operation & 255);
	}
	fLongValue = value;
}

void Condition::SetValue(double value, long operation)
{
	if (fDataType != kConditionDataTypeFloat)
		return;
	if (operation >= 0 && (operation & 255) <= kConditionOpGreater)
	{
		fOperation = (operation & kConditionOpNot) | (operation & 255);
	}
	fFloatValue = value;
}

void Condition::SetValue(string value, long operation)
{
	if (fDataType != kConditionDataTypeString)
		return;
	if (operation >= 0 && (operation & 255) <= kConditionOpContain)
	{
		fOperation = (operation & kConditionOpNot) | (operation & 255);
	}
	fStringValue = value;
}

bool Condition::Check(GameServer *server)
{
	unsigned long longVar;
	double floatVar;
	string stringVar;
	bool ret = false;

	switch (fType)
	{
	case kConditionTypeIP:
		longVar = server->fIP;
		break;
	case kConditionTypePort:
		longVar = server->fPort;
		break;
	case kConditionTypeCntPlayers:
		longVar = server->fCntPlayers;
		break;
	case kConditionTypeCustomLong:
		longVar = 0;
		break;
	case kConditionTypeCustomFloat:
		floatVar = 0.0;
		break;
	case kConditionTypeHostname:
		stringVar = server->fHostName;
		break;
	case kConditionTypeGame:
		stringVar = gGames[server->fGame].fID;
		break;
	case kConditionTypeGamedir:
		stringVar = server->fGameDir;
		break;
	case kConditionTypeGametype:
		stringVar = server->fGameTypeStr;
		break;
	case kConditionTypeMap:
		stringVar = server->fMapName;
		break;
	case kConditionTypePlayer:
		stringVar = "";
		break;
	case kConditionTypeCustomString:
		stringVar = "";
		break;
	}

	switch (fDataType)
	{
	case kConditionDataTypeLong:
		switch (fOperation & 255)
		{
		case kConditionOpEqual:
			ret = longVar == fLongValue;
			break;
		case kConditionOpLower:
			ret = longVar < fLongValue;
			break;
		case kConditionOpGreater:
			ret = longVar > fLongValue;
			break;
		}
		break;
	case kConditionDataTypeFloat:
		switch (fOperation & 255)
		{
		case kConditionOpEqual:
			ret = floatVar == fFloatValue;
			break;
		case kConditionOpLower:
			ret = floatVar < fFloatValue;
			break;
		case kConditionOpGreater:
			ret = floatVar > fFloatValue;
			break;
		}
		break;
	case kConditionDataTypeString:
		switch (fOperation & 255)
		{
		case kConditionOpEqual:
			ret = stringVar.compare(fStringValue) == 0;
			break;
		case kConditionOpLower:
			ret = stringVar.compare(fStringValue) < 0;
			break;
		case kConditionOpGreater:
			ret = stringVar.compare(fStringValue) > 0;
			break;
		case kConditionOpContain:
			ret = stringVar.find(fStringValue) != npos;
			break;
		}
		break;
	}

	if (fOperation & kConditionOpNot)
		return !ret;
	else
		return ret;
}
