/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "Parser.h"
#include "Game.h"

static const basic_string <char>::size_type npos = -1;

void Parser::ParseServerStatus(ServerStatus *state, GameServer *server)
{
	server->fFound = true;
	server->fPing = state->fPing;
	switch(server->fGame)
	{
	case kGameQW:
		ParseQuakeworldStatus(state->fStatus, server);
		break;
	case kGameQ2:
		ParseQuake2Status(state->fStatus, server);
		break;
	case kGameQ3:
	case kGameEF:
	case kGameFK2:
	case kGameRTCW:
	case kGameJK2:
	case kGameSOF2:
	case kGameMOHA:
	case kGameMOHS:
	case kGameMOHB:
	case kGameEF2:
	case kGameJK3:
	case kGameCOD:
	case kGameET:
		ParseQuake3Status(state->fStatus, server);
		break;
	case kGameHL:
		ParseHalfLifeStatus(state->fStatus, server);
		break;
	case kGameUT:
	case kGameBF42:
	case kGameBFVN:
		ParseUnrealTournamentStatus(state->fStatus, server);
		break;
	case kGameUT23:
	case kGameUT24:
		ParseUnrealTournament2003Status(state->fStatus, server);
		break;
	case kGameSOF:
		ParseSoldierOfFortuneStatus(state->fStatus, server);
		break;
	case kGameTBS:
		ParseTribesStatus(state->fStatus, server);
		break;
	case kGameTBS2:
		ParseTribes2Status(state->fStatus, server);
		break;
	case kGameFCRY:
		ParseFarCryStatus(state->fStatus, server);
		break;
	}
}

long Parser::GetGameFromStatus(string &status)
{
	long posStart;
	long posEnd;

	if (status.length() < 4)
		return -1;

	switch ((unsigned char) status[0])
	{
	case 0x09:
		return kGameFCRY;
	case 0x10:
	case 0x14:
		return kGameTBS2;
	case 0x5C:
		if (status.find("\\gamename\\bfield1942") != npos)
			return kGameBF42;
		if (status.find("\\game_id\\bfvietnam") != npos)
			return kGameBFVN;
		posStart = status.find("\\gamever\\");
		if (posStart != npos)
		{
			posStart += 9;
			posEnd = status.find("\\", posStart);
			if (posEnd != npos)
			{
				string num;
				long version;
				num = status.substr(posStart, posEnd - 1);
				version = atoi(num.c_str());
				if (version > 400 && version < 500)
					return kGameUT;
			}
		}
		break;
	case 0x61:
		return kGameTBS;
	case 0x78:
	case 0x79:
		return kGameUT23;
	case 0x7F:
		return kGameUT24;
	case 0xFE: //Half-Life multi part packet
		return kGameHL;
	case 0xFF:
		switch ((unsigned char) status[4])
		{
		case 0x01:
			if (status.find("\\version\\Medal") != npos)
				if (status.find("Spearhead") != npos)
					return kGameMOHS;
				else if (status.find("Breakthrough") != npos)
					return kGameMOHB;
				else
					return kGameMOHA;
			else
				return -1;
		case 0x6E:
			return kGameQW;
		case 0x70:
			if (status.find("\\version\\3.") != npos)
				return kGameQ2;
			else
				if (status.find("\\version\\1") != npos &&
					status.find("x86F\\") != npos)
					return kGameSOF;
				else
					return -1;
			break;
		case 0x69:
		case 0x73:
			if (status.find("\\version\\Q3") != npos ||
				status.find("\\version\\GTV") != npos)
				return kGameQ3;
			else if (status.find("\\version\\ST:V HM") != npos)
				return kGameEF;
			else if (status.find("\\version\\Heavy Metal: FAKK2") != npos)
				return kGameFK2;
			else if (status.find("\\version\\Wolf") != npos)
				return kGameRTCW;
			else if (status.find("\\version\\JK2MP") != npos)
				return kGameJK2;
			else if (status.find("\\version\\SOF2MP") != npos)
				return kGameSOF2;
			else if (status.find("\\version\\Elite Force II") != npos)
				return kGameEF2;
			else if (status.find("\\version\\JAmp") != npos)
				return kGameJK3;
			else if (status.find("\\shortversion\\") != npos && 
					 status.find("\\pswrd\\") != npos)
				return kGameCOD;
			else if (status.find("\\version\\ET") != npos)
				return kGameET;
			else
				return -1;
		case 0x43:
		case 0x44:
		case 0x45:
			return kGameHL;
		default:
			return -1;
		}
		break;
	}
	return -1;
}

bool Parser::CheckGameFromStatus(string &status, GameServer *server)
{
	string str;
	long posEnd;
	StringVariable *variable;

	if (server->fGame == kGameUT ||
		server->fGame == kGameBF42 ||
		server->fGame == kGameBFVN)
	{
		variable = server->GetVariable("queryid");
		if (variable == NULL)
			return false;
		posEnd = variable->fValue.find(".");
		if (posEnd != npos)
		{
			str = variable->fValue.substr(0, posEnd);
			if (status.find("\\queryid\\" + str) != npos)
				return true;
		}
	}
	return false;
}

long Parser::ParseVariables(long posBuf, string &status, GameServer *server)
{
	long posStart;
	long posEnd;
	StringVariable newVar;

//	server->fVariables.clear();
	while ((posStart = status.find("\\", posBuf)) != npos)
	{
		posStart++;
		posEnd = status.find("\\", posStart);
		if (posEnd == npos)
		{
			return -1;
		}
		newVar.fName = status.substr(posStart, posEnd - posStart);
		posStart = posEnd + 1;
		posEnd = status.find_first_of("\\\x00A", posStart);
		if (posEnd == npos)
		{
			return -1;
		}
		newVar.fValue = status.substr(posStart, posEnd - posStart);
		list<StringVariable>::iterator it;
		for (it = server->fVariables.begin(); it != server->fVariables.end(); it++)
		{
			if (it->fName == newVar.fName)
			{
				it->fValue = newVar.fValue;
				break;
			}
		}
		if (it == server->fVariables.end())
			server->fVariables.push_back(newVar);
		posBuf = posEnd;
	}
	return posBuf;
}

long Parser::ParsePlayers(long posBuf, string &status, GameServer *server, bool containsFrags)
{
	long posEnd;
	long index;
	Player player;
	string str;

	server->fCntPlayers = 0;
	server->fPlayers.clear();
	while (posBuf < (long) status.length())
	{
		if (containsFrags)
		{
			// Frags
			posEnd = status.find(" ", posBuf);
			if (posEnd == npos)
				return -1;
			str = status.substr(posBuf, posEnd - posBuf);
			player.fFrags = atol(str.c_str());
			posBuf = posEnd + 1;
		}
		// Ping
		posEnd = status.find(" ", posBuf);
		if (posEnd == npos)
			return -1;
		str = status.substr(posBuf, posEnd - posBuf);
		player.fPing = atol(str.c_str());
		posBuf = posEnd + 1;
		// Name
		posEnd = status.find("\x00A", posBuf);
		if (posEnd == npos)
			return -1;
		str = status.substr(posBuf + 1, posEnd - posBuf - 2);
		switch (server->fGame)
		{
		case kGameQ2:
		case kGameSOF:
			player.fName = "";
			for (index = 0; index < (long) str.length(); index++)
				player.fName += LetterCodeQuake2[str[index]];
			break;
		case kGameQ3:
			player.fName = "";
			for (index = 0; index < (long) str.length(); index++)
			{
				if (str[index] == '^')
				{
					if (index < (long) str.length() - 1 && str[index + 1] != '^')
					{
						index += 2;
					}
					player.fName += LetterCodeQuake3[str[index]];
				}
				else
				{
					player.fName += LetterCodeQuake3[str[index]];
				}
			}
			break;
		default:
			player.fName = str;
			break;
		}
		posBuf = posEnd + 1;
		server->fPlayers.push_back(player);
		server->fCntPlayers++;
	}
	return posBuf;
}

void Parser::ParseQuakeworldStatus(string &status, GameServer *server)
{
	long posBuf;
	long posEnd;
	long index;
	string str;
	Player player;

	// Variablen parsen
	posBuf = 5;
	posBuf = ParseVariables(posBuf, status, server);

	// Player parsen
	posBuf++;
	server->fCntPlayers = 0;
	server->fPlayers.clear();
	while (posBuf < (long) status.length())
	{
		// ID
		posEnd = status.find(" ", posBuf);
		if (posEnd == npos)
			break;
		posBuf = posEnd + 1;
		// Frags
		posEnd = status.find(" ", posBuf);
		if (posEnd == npos)
			break;
		str = status.substr(posBuf, posEnd - posBuf);
		player.fFrags = atol(str.c_str());
		posBuf = posEnd + 1;
		// Time
		posEnd = status.find(" ", posBuf);
		if (posEnd == npos)
			break;
		posBuf = posEnd + 1;
		// Ping
		posEnd = status.find(" ", posBuf);
		if (posEnd == npos)
			break;
		str = status.substr(posBuf, posEnd - posBuf);
		player.fPing = atol(str.c_str());
		posBuf = posEnd + 2;
			// Anfhrungsstriche berspringen
		// Name
		posEnd = status.find("\"", posBuf);
		if (posEnd == npos)
			break;
		str = status.substr(posBuf, posEnd - posBuf);
		player.fName = "";
		for (index = 0; index < (long) str.length(); index++)
			player.fName += LetterCodeQuakeworld[str[index]];
		posBuf = posEnd + 2;
		// Skin
		posEnd = status.find(" ", posBuf);
		if (posEnd == npos)
			break;
		posBuf = posEnd + 1;
		// Farbe 1
		posEnd = status.find(" ", posBuf);
		if (posEnd == npos)
			break;
		posBuf = posEnd + 1;
		// Farbe 2
		posEnd = status.find("\x00A", posBuf);
		if (posEnd == npos)
			break;
		posBuf = posEnd + 1;
		server->fPlayers.push_back(player);
		server->fCntPlayers++;
	}

	// server Objekt fllen
	list<StringVariable>::iterator i;
	long deathmatch = 0, teamplay = 0, ctf = 0;

	server->fGameDir = "id1";
	for (i = server->fVariables.begin(); i != server->fVariables.end(); i++)
	{
		if (i->fName == "hostname")
		{
			server->fHostName = i->fValue;
		}
		else if (i->fName == "*gamedir")
		{
			server->fGameDir = i->fValue;
		}
		else if (i->fName == "map")
		{
			server->fMapName = i->fValue;
		}
		else if (i->fName == "maxclients")
		{
			server->fMaxPlayers = atol(i->fValue.c_str());
		}
		else if (i->fName == "needpass")
		{
			if (atol(i->fValue.c_str()))
				server->fNeedPass = true;
			else
				server->fNeedPass = false;
		}
		else if (i->fName == "deathmatch")
		{
			deathmatch = atol(i->fValue.c_str());
		}
		else if (i->fName == "teamplay")
		{
			teamplay = atol(i->fValue.c_str());
		}
		else if (i->fName == "ctf")
		{
			ctf = atol(i->fValue.c_str());
		}
	}
}

void Parser::ParseQuake2Status(string &status, GameServer *server)
{
	long posBuf;

	// Variablen parsen
	posBuf = 10;
	posBuf = ParseVariables(posBuf, status, server);

	// Player parsen
	posBuf++;
	posBuf = ParsePlayers(posBuf, status, server, true);

	// server Objekt fllen
	list<StringVariable>::iterator i;
	long deathmatch = 0, ctf = 0;

	server->fGameDir = "baseq2";
	for (i = server->fVariables.begin(); i != server->fVariables.end(); i++)
	{
		if (i->fName == "hostname")
		{
			server->fHostName = i->fValue;
		}
		else if (i->fName == "gamedir")
		{
			server->fGameDir = i->fValue;
		}
		else if (i->fName == "mapname")
		{
			server->fMapName = i->fValue;
		}
		else if (i->fName == "maxclients")
		{
			server->fMaxPlayers = atol(i->fValue.c_str());
		}
		else if (i->fName == "dmflags")
		{
			server->fDMFlags = atol(i->fValue.c_str());
		}
		else if (i->fName == "needpass")
		{
			if (atol(i->fValue.c_str()))
				server->fNeedPass = true;
			else
				server->fNeedPass = false;
		}
		else if (i->fName == "deathmatch")
		{
			deathmatch = atol(i->fValue.c_str());
		}
		else if (i->fName == "ctf")
		{
			ctf = atol(i->fValue.c_str());
		}
	}
	if (ctf)
		server->fGameTypeStr = "CTF";
	else
		if (deathmatch)
			if ((server->fDMFlags & 64) || (server->fDMFlags & 128))
				server->fGameTypeStr = "TDM";
			else
				server->fGameTypeStr = "DM";
		else
			server->fGameTypeStr = "";
}

void Parser::ParseQuake3Status(string &status, GameServer *server)
{
	long posBuf;

	// Variablen parsen
	posBuf = 19;
	posBuf = ParseVariables(posBuf, status, server);

	// Player parsen
	posBuf++;
	if (status[4] == 0x01) // wenn Medal of Honor
		posBuf = ParsePlayers(posBuf, status, server, false);
	else
		posBuf = ParsePlayers(posBuf, status, server, true);
	// server Objekt fllen
	list<StringVariable>::iterator i;
	for (i = server->fVariables.begin(); i != server->fVariables.end(); i++)
	{
		if (i->fName == "sv_hostname" || i->fName == "hostname")
		{
			server->fHostName = i->fValue;
		}
		else if (i->fName == "gamename")
		{
			server->fGameDir = i->fValue;
		}
		else if (i->fName == "mapname")
		{
			server->fMapName = i->fValue;
		}
		else if (i->fName == "sv_maxclients")
		{
			server->fMaxPlayers = atol(i->fValue.c_str());
		}
		else if (i->fName == "dmflags")
		{
			server->fDMFlags = atol(i->fValue.c_str());
		}
		else if (i->fName == "g_needpass")
		{
			if (atol(i->fValue.c_str()))
				server->fNeedPass = true;
			else
				server->fNeedPass = false;
		}
		else if (i->fName == "pswrd")
		{
			if (atol(i->fValue.c_str()))
				server->fNeedPass = true;
			else
				server->fNeedPass = false;
		}
		else if (i->fName == "clients")
		{
			server->fCntPlayers = atol(i->fValue.c_str());
		}
		else if (i->fName == "mp_gametype" && server->fGame == kGameEF2)
		{
			server->fGameType = atol(i->fValue.c_str());
			switch (server->fGameType)
			{
			case 0:
				server->fGameTypeStr = "holomatch";
				break;
			case 1:
				server->fGameTypeStr = "team holomatch";
				break;
			case 2:
				server->fGameTypeStr = "capture the flag";
				break;
			case 3:
				server->fGameTypeStr = "bomb defusion";
				break;
			default:
				server->fGameTypeStr = "MOD";
			}
		}
		else if (i->fName == "g_gametype" && server->fGame != kGameEF2)
		{
			if (server->fGame == kGameCOD)
			{
				server->fGameType = 0;
				server->fGameTypeStr = i->fValue;
			}
			else
			{
				server->fGameType = atol(i->fValue.c_str());
				if (server->fGame == kGameET)
				{
					if (server->fGameType == 2)
						server->fGameTypeStr = "Single-Map Objective";
					else if (server->fGameType == 3)
						server->fGameTypeStr = "Stopwatch";
					else if (server->fGameType == 4)
						server->fGameTypeStr = "Campaign";
					else if (server->fGameType == 5)
						server->fGameTypeStr = "Last Man Standing";
				}
				else
				{
					switch (server->fGameType)
					{
					case kGameTypeFFA:
						server->fGameTypeStr = "FFA";
						break;
					case kGameType1V1:
						server->fGameTypeStr = "1V1";
						break;
					case kGameTypeSP:
						server->fGameTypeStr = "SP";
						break;
					case kGameTypeTDM:
						server->fGameTypeStr = "TDM";
						break;
					case kGameTypeCTF:
						server->fGameTypeStr = "CTF";
						break;
					default:
						server->fGameTypeStr = "MOD";
					}
				}
			}
		}
		else if (i->fName == "g_gametypestring")
		{
			server->fGameType = 0;
			server->fGameTypeStr = i->fValue;
		}
	}
}

void Parser::ParseHalfLifeStatus(string &status, GameServer *server)
{
	long posBuf;
	long posEnd;
	long index;
	StringVariable newVar;
	Player player;
	char buf[10];
	long cntVar;
	long deathmatch = 0, teamplay = 0;
	string zero;

	zero.assign("\x000", 1);
	posBuf = 4;
	switch (status[posBuf])
	{
	case 'C':
		server->fGotPacket[0] = true;
		posBuf++;
		// Adresse
		posEnd = status.find(zero, posBuf);
		if (posEnd == npos)
			return;
		newVar.fName = "address";
		newVar.fValue = status.substr(posBuf, posEnd - posBuf);
		server->SetVariable(newVar);
		posBuf = posEnd + 1;
		// Hostname
		posEnd = status.find(zero, posBuf);
		if (posEnd == npos)
			return;
		newVar.fName = "hostname";
		newVar.fValue = status.substr(posBuf, posEnd - posBuf);
		server->SetVariable(newVar);
		server->fHostName = newVar.fValue;
		posBuf = posEnd + 1;
		// Mapname
		posEnd = status.find(zero, posBuf);
		if (posEnd == npos)
			return;
		newVar.fName = "mapname";
		newVar.fValue = status.substr(posBuf, posEnd - posBuf);
		server->SetVariable(newVar);
		server->fMapName = newVar.fValue;
		posBuf = posEnd + 1;
		// Gamedir
		posEnd = status.find(zero, posBuf);
		if (posEnd == npos)
			return;
		newVar.fName = "gamedir";
		newVar.fValue = status.substr(posBuf, posEnd - posBuf);
		server->SetVariable(newVar);
		server->fGameDir = newVar.fValue;
		posBuf = posEnd + 1;
		// "Half-life"
		posEnd = status.find(zero, posBuf);
		if (posEnd == npos)
			return;
		newVar.fName = "gamename";
		newVar.fValue = status.substr(posBuf, posEnd - posBuf);
		server->SetVariable(newVar);
		posBuf = posEnd + 1;
		// cntplayers
		server->fCntPlayers = status[posBuf];
		sprintf(buf, "%ld", server->fCntPlayers);
		newVar.fName = "cntplayers";
		newVar.fValue = buf;
		server->SetVariable(newVar);
		posBuf++;
		// maxplayers
		server->fMaxPlayers = status[posBuf];
		sprintf(buf, "%ld", server->fMaxPlayers);
		newVar.fName = "maxplayers";
		newVar.fValue = buf;
		server->SetVariable(newVar);
		break;
	case 'D':
		server->fGotPacket[1] = true;
		posBuf++;
		server->fCntPlayers = status[posBuf];
		posBuf++;
		server->fPlayers.clear();
		for (index = 0; index < server->fCntPlayers; index++)
		{
			posBuf++;
			// Name
			posEnd = status.find(zero, posBuf);
			if (posEnd == npos)
				return;
			player.fName = status.substr(posBuf, posEnd - posBuf);
			posBuf = posEnd + 1;
			// Frags
			player.fFrags = 0;
			player.fFrags |= status[posBuf];
			player.fFrags |= status[posBuf + 1] << 8;
			player.fFrags |= status[posBuf + 2] << 16;
			player.fFrags |= status[posBuf + 3] << 24;
			posBuf += 4;
			// Time
			posBuf += 4;
			player.fPing = 0;
			server->fPlayers.push_back(player);
		}
		break;
	case 'E':
		server->fGotPacket[2] = true;
		posBuf++;
		cntVar = status[posBuf];
		posBuf++;
		posBuf++;
		for (index = 0; index < cntVar; index++)
		{
			posEnd = status.find(zero, posBuf);
			if (posEnd == npos)
				return;
			newVar.fName = status.substr(posBuf, posEnd - posBuf);
			posBuf = posEnd + 1;

			posEnd = status.find(zero, posBuf);
			if (posEnd == npos)
				return;
			newVar.fValue = status.substr(posBuf, posEnd - posBuf);
			int pos = 0;
			while ((pos = newVar.fValue.find_first_of("\n\r")) != -1)
				newVar.fValue.erase(pos, 1);
			posBuf = posEnd + 1;
			server->SetVariable(newVar);
		}
		list<StringVariable>::iterator i;
		for (i = server->fVariables.begin(); i != server->fVariables.end(); i++)
		{
			if (i->fName == "mp_teamplay")
			{
				teamplay = atol(i->fValue.c_str());
			}
			else if (i->fName == "deathmatch")
			{
				deathmatch = atol(i->fValue.c_str());
			}
			else if (i->fName == "sv_password")
			{
				if (atol(i->fValue.c_str()))
					server->fNeedPass = true;
				else
					server->fNeedPass = false;
			}
		}
/*		if (server->fGameDir.length() == 0)
			if (deathmatch)
				if (teamplay)
					server->fGameTypeStr = "TDM";
				else
					server->fGameTypeStr = "DM";
			else
				server->fGameTypeStr = "";
*/
		break;
	}
}

void Parser::ParseUnrealTournamentStatus(string &status, GameServer *server)
{
	long posBuf;
	long posEnd;
	StringVariable newVar;
	Player player;
	string str;
	long bugfix = 0;

	posBuf = 0;
	while (posBuf < (long) status.length() && status[posBuf] == '\\')
	{
		posBuf++;
		posEnd = status.find("\\", posBuf);
		if (posEnd == npos)
			break;
		newVar.fName = status.substr(posBuf, posEnd - posBuf);
		posBuf = posEnd + 1;
		posEnd = status.find("\\", posBuf);
		if (posEnd == npos)
		{
			newVar.fValue = status.substr(posBuf);
			server->SetVariable(newVar);
			break;
		}
		newVar.fValue = status.substr(posBuf, posEnd - posBuf);
		posBuf = posEnd;
		if (newVar.fName == "hostname")
		{
			server->fHostName = newVar.fValue;
			bugfix = 0;
		}
		else if (newVar.fName == "gamename")
		{
			server->fGameDir = newVar.fValue;
		}
		else if (newVar.fName == "gameId")
		{
			server->fGameDir = newVar.fValue;
		}
		else if (newVar.fName == "game_id")
		{
			server->fGameDir = newVar.fValue;
		}
		else if (newVar.fName == "mapname")
		{
			server->fMapName = newVar.fValue;
			bugfix = 3;
		}
		else if (newVar.fName == "gametype")
		{
			server->fGameTypeStr = newVar.fValue;
			bugfix = 0;
		}
		else if (newVar.fName == "hostport")
		{
			server->fPort = atoi(newVar.fValue.c_str());
			server->fPortStr = newVar.fValue;
			bugfix = 0;
		}
		else if (newVar.fName == "numplayers")
		{
			server->fCntPlayers = atoi(newVar.fValue.c_str());
			bugfix = 2;
		}
		else if (newVar.fName == "maxplayers")
		{
			server->fMaxPlayers = atoi(newVar.fValue.c_str());
			bugfix = 0;
		}
		else if (newVar.fName == "password")
		{
			str = (newVar.fValue.c_str());
			if (str == "TRUE" || str == "1")
				server->fNeedPass = true;
			else
				server->fNeedPass = false;
		}
		else if (newVar.fName.compare(0, 7, "player_") == 0)
		{
			str = newVar.fName.substr(7);
			long num = atoi(str.c_str());
			if (num >= (long) server->fPlayers.size())
				server->fPlayers.resize(num + 1);
			server->fPlayers[num].fName = newVar.fValue;
		}
		else if (newVar.fName.compare(0, 11, "playername_") == 0)
		{
			str = newVar.fName.substr(11);
			long num = atoi(str.c_str());
			if (num >= (long) server->fPlayers.size())
				server->fPlayers.resize(num + 1);
			server->fPlayers[num].fName = newVar.fValue;
		}
		else if (server->fGame == kGameUT && newVar.fName.compare(0, 6, "frags_") == 0)
		{
			str = newVar.fName.substr(6);
			long num = atoi(str.c_str());
			if (num >= (long) server->fPlayers.size())
				server->fPlayers.resize(num + 1);
			server->fPlayers[num].fFrags = atol(newVar.fValue.c_str());
		}
		else if (server->fGame != kGameUT && newVar.fName.compare(0, 6, "score_") == 0)
		{
			str = newVar.fName.substr(6);
			long num = atoi(str.c_str());
			if (num >= (long) server->fPlayers.size())
				server->fPlayers.resize(num + 1);
			server->fPlayers[num].fFrags = atol(newVar.fValue.c_str());
		}
		else if (newVar.fName.compare(0, 5, "ping_") == 0)
		{
			str = newVar.fName.substr(5);
			long num = atoi(str.c_str());
			if (num >= (long) server->fPlayers.size())
				server->fPlayers.resize(num + 1);
			server->fPlayers[num].fPing = atol(newVar.fValue.c_str());
		}
		else if (newVar.fName == "gamemode")
		{
			bugfix = 1;
		}
		else if (newVar.fName == "Spawn Wave Time")
		{
			bugfix = 5;
		}
		else if (newVar.fName == "Number Of Rounds")
		{
			bugfix = 0;
		}
		else if (server->fGame == kGameBF42 && bugfix == 1)
		{
			// maxplayers
			newVar.fName = "maxplayers";
			server->fMaxPlayers = atoi(newVar.fValue.c_str());
			bugfix = 0;
		}
		else if (server->fGame == kGameBF42 && bugfix == 2)
		{
			// gametype
			newVar.fName = "gametype";
			server->fGameTypeStr = newVar.fValue;
			bugfix = 0;
		}
		else if (server->fGame == kGameBF42 && bugfix == 3)
		{
			// hostport
			newVar.fName = "hostport";
			server->fPort = atoi(newVar.fValue.c_str());
			server->fPortStr = newVar.fValue;
			bugfix = 4;
		}
		else if (server->fGame == kGameBF42 && bugfix == 4)
		{
			// hostname
			newVar.fName = "hostname";
			server->fHostName = newVar.fValue;
			bugfix = 0;
		}
		else if (server->fGame == kGameBF42 && bugfix == 5)
		{
			// hostname
			newVar.fName = "Number Of Rounds";
			bugfix = 0;
		}
		server->SetVariable(newVar);
	}
	int cnt = server->fPlayers.size() - server->fCntPlayers;
	for (int i = 0; i < cnt; i++)
		server->fPlayers.pop_back();
}

void Parser::ParseUnrealTournament2003Status(string &status, GameServer *server)
{
	long posBuf;
	long len;
	unsigned long value;
	char buf[10];
	StringVariable newVar;
	Player newPlayer;

	posBuf = 4;
	switch (status[posBuf])
	{
	case 0x00:
		// server info
		posBuf += 1;
		posBuf += 5;
		// server port
		value = (unsigned long) status[posBuf];
		value |= (unsigned long) (status[posBuf + 1] << 8);
		value |= (unsigned long) (status[posBuf + 2] << 16);
		value |= (unsigned long) (status[posBuf + 3] << 24);
		server->SetAddress(server->fIP, (unsigned short) value, server->fQueryPort);
		newVar.fName = "hostport";
		newVar.fValue = server->fPortStr;
		server->SetVariable(newVar);
		posBuf += 4;
		posBuf += 4;
		// server name
		len = status[posBuf];
		posBuf += 1;
		server->fHostName = status.substr(posBuf, len - 1);
		newVar.fName = "hostname";
		newVar.fValue = server->fHostName;
		server->SetVariable(newVar);
		posBuf += len;
		// map name
		len = status[posBuf];
		posBuf += 1;
		server->fMapName = status.substr(posBuf, len - 1);
		newVar.fName = "mapname";
		newVar.fValue = server->fMapName;
		server->SetVariable(newVar);
		posBuf += len;
		// gametype
		len = status[posBuf];
		posBuf += 1;
		server->fGameTypeStr = status.substr(posBuf, len - 1);
		newVar.fName = "gametype";
		newVar.fValue = server->fGameTypeStr;
		server->SetVariable(newVar);
		posBuf += len;
		// numplayers
		value = (unsigned long) status[posBuf];
		value |= (unsigned long) (status[posBuf + 1] << 8);
		value |= (unsigned long) (status[posBuf + 2] << 16);
		value |= (unsigned long) (status[posBuf + 3] << 24);
		server->fCntPlayers = value;
		sprintf(buf, "%ld", value);
		newVar.fName = "numplayers";
		newVar.fValue = buf;
		server->SetVariable(newVar);
		posBuf += 4;
		// maxplayers
		value = (unsigned long) status[posBuf];
		value |= (unsigned long) (status[posBuf + 1] << 8);
		value |= (unsigned long) (status[posBuf + 2] << 16);
		value |= (unsigned long) (status[posBuf + 3] << 24);
		server->fMaxPlayers = value;
		sprintf(buf, "%ld", value);
		newVar.fName = "maxplayers";
		newVar.fValue = buf;
		server->SetVariable(newVar);
		posBuf += 4;
		break;
	case 0x01:
		// game info
		posBuf += 1;
		while (posBuf < (long) status.length() && status[posBuf] != 0x00)
		{
			len = status[posBuf];
			posBuf += 1;
			newVar.fName = status.substr(posBuf, len - 1);
			posBuf += len;
			len = status[posBuf];
			posBuf += 1;
			if (len > 0)
			{
				newVar.fValue = status.substr(posBuf, len - 1);
				posBuf += len;
			}
			else
			{
				newVar.fValue = "";
			}
			server->SetVariable(newVar);
		}
		break;
	case 0x02:
		// player info
		posBuf += 1;
		server->fCntPlayers = 0;
		server->fPlayers.clear();
		while (posBuf < (long) status.length() && status[posBuf] != 0x00)
		{
			// player id
			posBuf += 4;
			// player name
			len = status[posBuf];
			posBuf += 1;
			newPlayer.fName = status.substr(posBuf, len - 1);
			posBuf += len;
			// player ping
			value = 0;
			value |= (unsigned char) status[posBuf];
			value |= ((unsigned char) status[posBuf + 1]) << 8;
			value |= ((unsigned char) status[posBuf + 2]) << 16;
			value |= ((unsigned char) status[posBuf + 3]) << 24;
			newPlayer.fPing = value;
			posBuf += 4;
			// player score
			value = 0;
			value |= (unsigned char) status[posBuf];
			value |= ((unsigned char) status[posBuf + 1]) << 8;
			value |= ((unsigned char) status[posBuf + 2]) << 16;
			value |= ((unsigned char) status[posBuf + 3]) << 24;
			newPlayer.fFrags = value;
			posBuf += 4;
			// player stats id
			posBuf += 4;
			server->fPlayers.push_back(newPlayer);
			server->fCntPlayers++;
		}
		break;
	}
}

void Parser::ParseSoldierOfFortuneStatus(string &status, GameServer *server)
{
	long posBuf;

	// Variablen parsen
	posBuf = 10;
	posBuf = ParseVariables(posBuf, status, server);

	// Player parsen
	posBuf++;
	posBuf = ParsePlayers(posBuf, status, server, true);

	// server Objekt fllen
	list<StringVariable>::iterator i;
	long deathmatch = 0, ctf = 0;

	for (i = server->fVariables.begin(); i != server->fVariables.end(); i++)
	{
		if (i->fName == "hostname")
		{
			server->fHostName = i->fValue;
		}
		else if (i->fName == "gamename")
		{
			server->fGameDir = i->fValue;
		}
		else if (i->fName == "mapname")
		{
			server->fMapName = i->fValue;
		}
		else if (i->fName == "maxclients")
		{
			server->fMaxPlayers = atol(i->fValue.c_str());
		}
		else if (i->fName == "dmflags")
		{
			server->fDMFlags = atol(i->fValue.c_str());
		}
		else if (i->fName == "needpass")
		{
			if (atol(i->fValue.c_str()))
				server->fNeedPass = true;
			else
				server->fNeedPass = false;
		}
		else if (i->fName == "deathmatch")
		{
			deathmatch = atol(i->fValue.c_str());
		}
		else if (i->fName == "ctf")
		{
			ctf = atol(i->fValue.c_str());
		}
	}
	if (ctf)
		server->fGameTypeStr = "CTF";
	else
		if (deathmatch)
			if ((server->fDMFlags & 64) || (server->fDMFlags & 128))
				server->fGameTypeStr = "TDM";
			else
				server->fGameTypeStr = "DM";
		else
			server->fGameTypeStr = "";
}

void Parser::ParseTribesStatus(string &status, GameServer *server)
{
	long posBuf;
	long len;
	long teams;
	long index;
	StringVariable newVar;
	string str;
	char buf[10];
	Player player;

	server->fVariables.clear();
	posBuf = 4;
	// "Tribes"
	len = status[posBuf];
	posBuf += len + 1;
	// Versionsnummer
	len = status[posBuf];
	newVar.fName = "version";
	newVar.fValue = status.substr(posBuf + 1, len);
	server->fVariables.push_back(newVar);
	posBuf += len + 1;
	// Hostname
	len = status[posBuf];
	newVar.fName = "hostname";
	newVar.fValue = status.substr(posBuf + 1, len);
	server->fVariables.push_back(newVar);
	server->fHostName = newVar.fValue;
	posBuf += len + 1;
	// dedicated ??
	newVar.fName = "dedicated";
	if (status[posBuf])
        newVar.fValue = "1";
	else
		newVar.fName = "0";
	server->fVariables.push_back(newVar);
	posBuf++;
	// password ??
	newVar.fName = "needpass";
	if (status[posBuf])
	{
        newVar.fValue = "1";
		server->fNeedPass = true;
	}
	else
	{
		newVar.fName = "0";
		server->fNeedPass = false;
	}
	server->fVariables.push_back(newVar);
	posBuf++;
	// Anzahl Spieler
	server->fCntPlayers = status[posBuf];
	newVar.fName = "cntplayers";
	sprintf(buf, "%ld", server->fCntPlayers);
	newVar.fValue = buf;
	server->fVariables.push_back(newVar);
	posBuf++;
	// Maximale Anzahl Spieler
	server->fMaxPlayers = status[posBuf];
	newVar.fName = "maxplayers";
	sprintf(buf, "%ld", server->fMaxPlayers);
	newVar.fValue = buf;
	server->fVariables.push_back(newVar);
	posBuf++;
	// ??? CPU-Speed ???
	posBuf += 2;
	// Gamedir
	len = status[posBuf];
	newVar.fName = "gamedir";
	newVar.fValue = status.substr(posBuf + 1, len);
	server->fVariables.push_back(newVar);
	server->fGameDir = newVar.fValue;
	posBuf += len + 1;
	// Gametype
	len = status[posBuf];
	newVar.fName = "gametype";
	newVar.fValue = status.substr(posBuf + 1, len);
	server->fVariables.push_back(newVar);
	server->fGameTypeStr = newVar.fValue;
	posBuf += len + 1;
	// Mapname
	len = status[posBuf];
	newVar.fName = "mapname";
	newVar.fValue = status.substr(posBuf + 1, len);
	server->fVariables.push_back(newVar);
	server->fMapName = newVar.fValue;
	posBuf += len + 1;
	// Description/Admin/Email
	len = status[posBuf];
	newVar.fName = "description";
	newVar.fValue = status.substr(posBuf + 1, len);
	server->fVariables.push_back(newVar);
	posBuf += len + 1;
	// Anzahl Teams
	teams = status[posBuf];
	newVar.fName = "teams";
	sprintf(buf, "%ld", teams);
	newVar.fValue = buf;
	server->fVariables.push_back(newVar);
	posBuf++;
	// First Title
	len = status[posBuf];
	posBuf += len + 1;
	// Second Title
	len = status[posBuf];
	posBuf += len + 1;
	// Teams
	for (index = 0; index < teams; index++)
	{
		// Team Name
		len = status[posBuf];
		posBuf += len + 1;
		// Team Points
		len = status[posBuf];
		posBuf += len + 1;
	}
	// Spieler
	server->fPlayers.clear();
	for (index = 0; index < server->fCntPlayers; index++)
	{
		// Ping
		player.fPing = (long) status[posBuf] << 2;
		posBuf++;
		// Packet Loss
		posBuf++;
		// Teamnummer
		posBuf++;
		// Spielername
		len = status[posBuf];
		player.fName = status.substr(posBuf + 1, len);
		posBuf += len + 1;
		// Frags
		len = status[posBuf];
		if (teams > 1)
			str = status.substr(posBuf + 7, 3);
		else
			str = status.substr(posBuf + 4, 2);
		player.fFrags = atoi (str.c_str());
		posBuf += len + 1;
		server->fPlayers.push_back(player);
	}
}

void Parser::ParseTribes2Status(string &status, GameServer *server)
{
	long posBuf;
	long len;
	long pktlen;
	StringVariable newVar;
	char buf[10];
	string str;
	long posStart;
	long posEnd;
	long teams;
	long index;
	long players;
	bool bot;
	Player player;

	if (str[0] != 20)
		return;
	server->fVariables.clear();
	posBuf = 6;
	// Gamedir
	len = status[posBuf];
	newVar.fName = "gamedir";
	newVar.fValue = status.substr(posBuf + 1, len);
	server->fVariables.push_back(newVar);
	server->fGameDir = newVar.fValue;
	posBuf += len + 1;
	// Gametype
	len = status[posBuf];
	newVar.fName = "gametype";
	newVar.fValue = status.substr(posBuf + 1, len);
	server->fVariables.push_back(newVar);
	server->fGameTypeStr = newVar.fValue;
	posBuf += len + 1;
	// Mapname
	len = status[posBuf];
	newVar.fName = "mapname";
	newVar.fValue = status.substr(posBuf + 1, len);
	server->fVariables.push_back(newVar);
	server->fMapName = newVar.fValue;
	posBuf += len + 1;
	// Status
	newVar.fName = "needpass";
	if (status[posBuf] & 2)	// TRIBES2_STATUS_PASSWORD
	{
        newVar.fValue = "1";
		server->fNeedPass = true;
	}
	else
	{
		newVar.fName = "0";
		server->fNeedPass = false;
	}
	server->fVariables.push_back(newVar);
	posBuf++;
	// NumPlayers
	server->fCntPlayers = status[posBuf];
	newVar.fName = "cntplayers";
	sprintf(buf, "%ld", server->fCntPlayers);
	newVar.fValue = buf;
	server->fVariables.push_back(newVar);
	posBuf++;
	// MaxPlayers
	server->fMaxPlayers = status[posBuf];
	newVar.fName = "maxplayers";
	sprintf(buf, "%ld", server->fMaxPlayers);
	newVar.fValue = buf;
	server->fVariables.push_back(newVar);
	posBuf++;
	// BotCount
	posBuf++;
	// CPU Speed
	posBuf += 2;
	// Hostname
	len = status[posBuf];
	newVar.fName = "hostname";
	newVar.fValue = status.substr(posBuf + 1, len);
	server->fVariables.push_back(newVar);
	server->fHostName = newVar.fValue;
	posBuf += len + 1;
	// Restlnge des Pakets
	pktlen = (unsigned char) status[posBuf];
	posBuf++;
	pktlen |= (((long) status[posBuf]) << 8);
	posBuf++;

	posStart = posBuf;
    posEnd = status.find("\x00A", posBuf);
	if (posEnd == npos)
		return;

	// Anzahl Teams
	str = status.substr(posBuf, posEnd - posBuf);
	teams = atoi(str.c_str());
	newVar.fName = "teams";
	newVar.fValue = str;
	server->fVariables.push_back(newVar);

	posBuf = posEnd + 1;
	// Teams
	for (index = 0; index < teams; index++)
	{
	    posEnd = status.find("\x009", posBuf);
		if (posEnd == npos)
		{
			teams = index;
			return;
		}
		posBuf = posEnd + 1;
	    posEnd = status.find("\x00A", posBuf);
		if (posEnd == npos)
		{
			teams = index;
			return;
		}
		posBuf = posEnd + 1;
	}
    posEnd = status.find("\x00A", posBuf);
	if (posEnd == npos || posEnd - posStart >= pktlen)
		return;
	// Anzahl Spieler
	str = status.substr(posBuf, posEnd - posBuf);
	players = atoi(str.c_str());
	posBuf = posEnd + 1;
	// Spieler
	server->fPlayers.clear();
	for (index = 0; index < players && posBuf - posStart < pktlen; index++)
	{
		posBuf++;
		if (status[posBuf] == 0x0E)
			bot = true;
		else
			bot = false;
		posBuf++;
	    posEnd = status.find("\x011", posBuf);
		if (posEnd == npos || posEnd - posStart >= pktlen)
			break;
		// Spieler-Name
		player.fName = status.substr(posBuf, posEnd - posBuf);
		if (bot)
			player.fName += " (Bot)";
		posBuf = posEnd + 1;
		posBuf++;
		if (posBuf - posStart >= pktlen)
			break;
	    posEnd = status.find("\x009", posBuf);
		if (posEnd == npos || posEnd - posStart >= pktlen)
			break;
		// Team des Spielers
		posBuf = posEnd + 1;
	    posEnd = status.find("\x00A", posBuf);
		if (posEnd == npos || posEnd - posStart >= pktlen)
			break;
		// Spieler-Frags
		str = status.substr(posBuf, posEnd - posBuf);
		player.fFrags = atoi (str.c_str());
		posBuf = posEnd + 1;
		server->fPlayers.push_back(player);
	}
}

string Parser::DecodeFarCryPacket(string & in)
{
	unsigned char al, bl, cl, dl;
	string out = "";

	dl = 8 - 1;
	bl = in[0];
	for (int i = 1; i < 2; i++)
	{
		al = in[i];
		cl = dl;
		al = al >> cl;
		cl = 1;
		bl = bl << cl;
		al = al | bl;
		bl = in[i];
		out += al;
	}

	dl = 8 - 2;
	bl = in[1];
	for (int i = 2; i < (int) in.length(); i++)
	{
		al = in[i];
		cl = dl;
		al = al >> cl;
		cl = 2;
		bl = bl << cl;
		al = al | bl;
		bl = in[i];
		out += al;
	}
	return out;
}

struct tCode
{
	unsigned char character;
	int length;
	int bits;
};

tCode codes[] = 
{
	0x20,  3, 0x007, // SPACE
	0x21, 10, 0x2E3, // !
	0x22, 10, 0x2E4, // 
	0x23, 10, 0x2E5, // #
	0x24, 10, 0x2E6, // 
	0x25, 10, 0x2E7, // %
	0x26, 10, 0x2E8, // &
	0x27, 10, 0x2E9, // '
	0x28, 10, 0x2EA, // (
	0x29, 10, 0x2EB, // )
	0x2A, 10, 0x2EC, // *
	0x2B, 10, 0x2ED, // +
	0x2C, 10, 0x2EE, // ,
	0x2D, 10, 0x2EF, // -
	0x2E,  8, 0x02D, // .
	0x2F, 10, 0x2F0, // /
	0x30,  4, 0x008, // 0
	0x31,  4, 0x006, // 1
	0x32,  5, 0x004, // 2
	0x33,  8, 0x0A0, // 3
	0x34,  5, 0x003, // 4
	0x35,  5, 0x000, // 5
	0x36,  7, 0x00A, // 6
	0x37,  9, 0x0B3, // 7
	0x38,  9, 0x146, // 8
	0x39,  9, 0x147, // 9
	0x3A, 10, 0x2F1, // :
	0x3B, 10, 0x2F2, // ;
	0x3C, 10, 0x2F3, // <
	0x3D, 10, 0x2F4, // =
	0x3E, 10, 0x2F5, // >
	0x3F, 10, 0x2F6, // ?
	0x40, 10, 0x2F7, // @
	0x41,  6, 0x002, // A
	0x42,  9, 0x148, // B
	0x43,  6, 0x024, // C
	0x44,  8, 0x0A1, // D
	0x45,  9, 0x149, // E
	0x46,  6, 0x003, // F
	0x47,  6, 0x00A, // G
	0x48,  9, 0x14A, // H
	0x49,  8, 0x058, // I
	0x4A,  9, 0x14B, // J
	0x4B,  9, 0x14C, // K
	0x4C,  7, 0x017, // L
	0x4D,  9, 0x14D, // M
	0x4E,  9, 0x14E, // N
	0x4F,  8, 0x0A2, // O
	0x50,  5, 0x00F, // P
	0x51,  9, 0x14F, // Q
	0x52,  5, 0x00E, // R
	0x53,  7, 0x04C, // S
	0x54,  6, 0x025, // T
	0x55,  7, 0x028, // U
	0x56,  9, 0x150, // V
	0x57,  9, 0x151, // W
	0x58,  6, 004, // X
	0x59,  9, 0x152, // Y
	0x5A,  9, 0x153, // Z
	0x5B, 10, 0x2F8, // [
	0x5C, 10, 0x2F9, // BACKSLASH
	0x5D, 10, 0x2FA, // ]
	0x5E, 10, 0x2FB, // ^
	0x5F, 10, 0x2FC, // _
	0x60, 10, 0x2FD, // `
	0x61,  6, 0x015, // a
	0x62,  9, 0x154, // b
	0x63,  7, 0x02D, // c
	0x64,  7, 0x02E, // d
	0x65,  7, 0x04D, // e
	0x66,  9, 0x155, // f
	0x67,  9, 0x156, // g
	0x68,  9, 0x157, // h
	0x69,  7, 0x029, // i
	0x6A,  9, 0x158, // j
	0x6B,  9, 0x159, // k
	0x6C,  9, 0x15A, // l
	0x6D,  9, 0x15B, // m
	0x6E,  7, 0x04E, // n
	0x6F,  7, 0x00B, // o
	0x70,  7, 0x02F, // p
	0x71,  9, 0x15C, // q
	0x72,  6, 0x012, // r
	0x73,  6, 0x013, // s
	0x74,  5, 0x008, // t
	0x75,  7, 0x04F, // u
	0x76,  9, 0x15D, // v
	0x77,  9, 0x15E, // w
	0x78,  9, 0x15F, // x
	0x79,  9, 0x160, // y
	0x7A,  9, 0x161, // z
	0x7B, 10, 0x2FE, // {
	0x7C, 10, 0x2FF, // |
	0x7D, 10, 0x300, // }
	0x7E, 10, 0x301, // ~
	0x7F, 10, 0x302, // 
	0x80, 10, 0x303, // 
	0x81, 10, 0x304, // 
	0x82, 10, 0x305, // 
	0x83, 10, 0x306, // 
	0x84, 10, 0x307, // 
	0x85, 10, 0x308, // 
	0x86, 10, 0x309, // 
	0x87, 10, 0x30A, // 
	0x88, 10, 0x30B, // 
	0x89, 10, 0x30C, // 
	0x8A, 10, 0x30D, // 
	0x8B, 10, 0x30E, // 
	0x8C, 10, 0x30F, // 
	0x8D, 10, 0x310, // 
	0x8E, 10, 0x311, // 
	0x8F, 10, 0x312, // 
	0x90, 10, 0x313, // 
	0x91, 10, 0x314, // 
	0x92, 10, 0x315, // 
	0x93, 10, 0x316, // 
	0x94, 10, 0x317, // 
	0x95, 10, 0x318, // 
	0x96, 10, 0x319, // 
	0x97, 10, 0x31A, // 
	0x98, 10, 0x31B, // 
	0x99, 10, 0x31C, // 
	0x9A, 10, 0x31D, // 
	0x9B, 10, 0x31E, // 
	0x9C, 10, 0x31F, // 
	0x9D, 10, 0x320, // 
	0x9E, 10, 0x321, // 
	0x9F, 10, 0x322, // 
	0xA0, 10, 0x323, // DAUER LEERZEICHEN
	0xA1, 10, 0x324, // 
	0xA2, 10, 0x325, // 
	0xA3, 10, 0x326, // 
	0xA4, 10, 0x327, // 
	0xA5, 10, 0x328, // 
	0xA6, 10, 0x329, // 
	0xA7, 10, 0x32A, // 
	0xA8, 10, 0x32B, // 
	0xA9, 10, 0x32C, // 
	0xAA, 10, 0x32D, // 
	0xAB, 10, 0x32E, // 
	0xAC, 10, 0x32F, // 
	0xAD, 10, 0x330, // 
	0xAE, 10, 0x331, // 
	0xAF, 10, 0x332, // 
	0xB0, 10, 0x333, // 
	0xB1, 10, 0x334, // 
	0xB2, 10, 0x335, // 
	0xB3, 10, 0x336, // 
	0xB4, 10, 0x337, // 
	0xB5, 10, 0x338, // 
	0xB6, 10, 0x339, // 
	0xB7, 10, 0x33A, // 
	0xB8, 10, 0x33B, // 
	0xB9, 10, 0x33C, // 
	0xBA, 10, 0x33D, // 
	0xBB, 10, 0x33E, // 
	0xBC, 10, 0x33F, // 
	0xBD, 10, 0x340, // 
	0xBE, 10, 0x341, // 
	0xBF, 10, 0x342, // 
	0xC0, 10, 0x343, // 
	0xC1, 10, 0x344, // 
	0xC2, 10, 0x345, // 
	0xC3, 10, 0x346, // 
	0xC4, 10, 0x347, // 
	0xC5, 10, 0x348, // 
	0xC6, 10, 0x349, // 
	0xC7, 10, 0x34A, // 
	0xC8, 10, 0x34B, // 
	0xC9, 10, 0x34C, // 
	0xCA, 10, 0x34D, // 
	0xCB, 10, 0x34E, // 
	0xCC, 10, 0x34F, // 
	0xCD, 10, 0x350, // 
	0xCE, 10, 0x351, // 
	0xCF, 10, 0x352, // 
	0xD0, 10, 0x353, // 
	0xD1, 10, 0x354, // 
	0xD2, 10, 0x355, // 
	0xD3, 10, 0x356, // 
	0xD4, 10, 0x357, // 
	0xD5, 10, 0x358, // 
	0xD6, 10, 0x359, // 
	0xD7, 10, 0x35A, // 
	0xD8, 10, 0x35B, // 
	0xD9, 10, 0x35C, // 
	0xDA, 10, 0x35D, // 
	0xDB, 10, 0x35E, // 
	0xDC, 10, 0x35F, // 
	0xDD, 10, 0x360, // 
	0xDE, 10, 0x361, // 
	0xDF, 10, 0x362, // 
	0xE0, 10, 0x363, // 
	0xE1, 10, 0x364, // 
	0xE2, 10, 0x365, // 
	0xE3, 10, 0x366, // 
	0xE4, 10, 0x367, // 
	0xE5, 10, 0x368, // 
	0xE6, 10, 0x369, // 
	0xE7, 10, 0x36A, // 
	0xE8, 10, 0x36B, // 
	0xE9, 10, 0x36C, // 
	0xEA, 10, 0x36D, // 
	0xEB, 10, 0x36E, // 
	0xEC, 10, 0x36F, // 
	0xED, 10, 0x370, // 
	0xEE, 10, 0x371, // 
	0xEF, 10, 0x372, // 
	0xF0, 10, 0x373, // 
	0xF1, 10, 0x374, // 
	0xF2, 10, 0x375, // 
	0xF3, 10, 0x376, // 
	0xF4, 10, 0x377, // 
	0xF5, 10, 0x378, // 
	0xF6, 10, 0x379, // 
	0xF7, 10, 0x37A, // 
	0xF8, 10, 0x37B, // 
	0xF9, 10, 0x37C, // 
	0xFA, 10, 0x37D, // 
	0xFB, 10, 0x37E, // 
	0xFC, 10, 0x37F, // 
	0xFD,  9, 0x058, // 
	0xFE,  9, 0x059, // 
	0xFF,  9, 0x0B2, // 
	0, 0, 0x000
};

string Parser::ReadFarCryString(cBitStream & s)
{
	string str;
	int i = 0;
	while (s.NextBits(4) != 0x03)
	{
		for (int j = 0; codes[j].character; j++)
		{
			if (s.NextBits(codes[j].length) == codes[j].bits)
			{
				str += codes[j].character;
				s.SeekBits(codes[j].length);
				break;
			}
		}
		if (!codes[j].character)
			return "";
		i++;
	}
	s.SeekBits(4);
	return str;
}

void Parser::ParseFarCryStatus(string &status, GameServer *server)
{
	StringVariable newVar;
	char tmp[10];

	string packet = DecodeFarCryPacket(status);
	cBitStream s((unsigned char *) packet.c_str(), packet.length(), true);
	// unknown
	s.SeekBytes(1);
	// packet length in bits
	s.SeekBytes(2);
	// unknown
	s.SeekBytes(1);
	// hostname
	server->fHostName = ReadFarCryString(s);
	newVar.fName = "sv_name";
	newVar.fValue = server->fHostName;
	server->SetVariable(newVar);
	// mapname
	server->fMapName = ReadFarCryString(s);
	newVar.fName = "mapname";
	newVar.fValue = server->fMapName;
	server->SetVariable(newVar);
	// game port
	server->fPort = ntohs(s.GetBits(16));
	sprintf(tmp, "%u", server->fPort);
	server->fPortStr = tmp;
	// mod
	server->fGameDir = ReadFarCryString(s);
	newVar.fName = "gamedir";
	newVar.fValue = server->fGameDir;
	server->SetVariable(newVar);
	// gametype
	server->fGameTypeStr = ReadFarCryString(s);
	server->fGameType = 0;
	newVar.fName = "g_gametype";
	newVar.fValue = server->fGameTypeStr;
	server->SetVariable(newVar);
	// numplayers
	server->fCntPlayers = s.GetBits(8);
	newVar.fName = "numplayers";
	sprintf(tmp, "%d", server->fCntPlayers);
	newVar.fValue = tmp;
	server->SetVariable(newVar);
	// maxplayers
	server->fMaxPlayers = s.GetBits(8);
	newVar.fName = "sv_maxplayers";
	sprintf(tmp, "%d", server->fMaxPlayers);
	newVar.fValue = tmp;
	server->SetVariable(newVar);
	// password
	server->fNeedPass = s.GetBits(1) ? true : false;
	newVar.fName = "sv_password";
	sprintf(tmp, "%d", server->fNeedPass);
	newVar.fValue = tmp;
	server->SetVariable(newVar);
}
